@@ -1,6 +1,6 @@
 # vim:cindent:ts=2:sw=2:et:fdm=marker:cms=\ #\ %s
 #
-# $Id: Document.pm 98 2007-12-02 13:18:47Z robert $
+# $Id: Document.pm 134 2008-04-21 21:30:30Z robert $
 #
 
 package Debian::DocBase::Document;
@@ -10,13 +10,36 @@ use warnings;
 
 use Debian::DocBase::Common;
 use Debian::DocBase::Utils;
-use Debian::DocBase::DocBaseFile qw(PARSE_FULL PARSE_GETDOCID);
+use Debian::DocBase::DocBaseFile;
+use Debian::DocBase::DB;
 use Carp;
 #use Scalar::Util qw(weaken);
 
 our %DOCUMENTS = ();
 my %section_map = ();
 
+#################################################
+###        PUBLIC STATIC FUNCTIONS            ###
+#################################################
+
+# return list of all proceseed documents
+sub GetDocumentList() { # {{{
+  return values %DOCUMENTS;
+} # }}}
+
+# check if $docid exists in status database
+sub IsRegistered($) { # {{{
+  my $docid = shift;
+  return Debian::DocBase::DB::GetStatusDB()->Exists($docid);
+} # }}}
+
+# return all documents id from status database
+sub GetAllRegisteredDocumentIDs() { # {{{
+  my $db    = Debian::DocBase::DB::GetStatusDB()->GetDB();
+  my @result = sort keys %$db;
+  return @result;
+} # }}}
+
 sub new { # {{{
     my $class      = shift;
     my $documentId = shift;
@@ -32,94 +55,71 @@ sub new { # {{{
         INVALID           => 1
     };
     bless($self, $class);
-    $self->_read_status_file($documentId);
+    $self->_ReadStatusDB($documentId);
     $DOCUMENTS{$documentId} = $self;
 #  weaken $DOCUMENTS{$documentId};
     return $self;
 } # }}}
 
+#################################################
+###            PUBLIC FUNCTIONS               ###
+#################################################
+
 sub DESTROY { # {{{
   my $self = shift;
-  delete $DOCUMENTS{$self->document_id()};
+  delete $DOCUMENTS{$self->GetDocumentID()};
 } # }}}
 
-# class function: return list of all proceseed documents
-sub GetDocumentList() { # {{{
-  return values %DOCUMENTS;
-} # }}}
-
-sub document_id() { # {{{
+sub GetDocumentID() { # {{{
   my $self = shift;
   return $self->{'DOCUMENT_ID'};
 } # }}}
 
-sub invalid() { # {{{
+sub Invalid() { # {{{
   my $self = shift;
   return $self->{'INVALID'};
 } # }}}
 
-# dies with Internal error if document hasn't been merged yet
-sub _check_merged($) { # {{{
-  my $self = shift;
-
-  carp "Internal error: Document " . $self->document_id(). " not yet merged"
-    unless $self->{'MERGED_CTRL_FILES'};
-} # }}}
-
-# returns $fld from $self->{'MAIN_DATA'}
-sub _get_main_fld($$) { # {{{
-  my $self = shift;
-  my $fld  = shift;
-
-  $self->_check_merged();
-
-  return "" if $self->invalid();
-
-  return "" unless $self->{'MAIN_DATA'}->{$fld};
-
-  return $self->{'MAIN_DATA'}->{$fld};
-} # }}}
-
 
 # getters for common fields
-sub abstract() { # {{{
+sub GetAbstract() { # {{{
   my $self = shift;
-  return $self->_get_main_fld($FLD_ABSTRACT);
+  return $self->_GetMainFld($FLD_ABSTRACT);
 } # }}}
 
-sub title() { # {{{
+sub GetTitle() { # {{{
   my $self = shift;
-  return $self->_get_main_fld($FLD_TITLE);
+  return $self->_GetMainFld($FLD_TITLE);
 } # }}}
 
-sub section() { # {{{
+sub GetSection() { # {{{
   my $self = shift;
-  return $self->_get_main_fld($FLD_SECTION);
+  return $self->_GetMainFld($FLD_SECTION);
 } # }}}
 
-sub author() { # {{{
+sub GetAuthor() { # {{{
   my $self = shift;
-  return $self->_get_main_fld($FLD_AUTHOR);
+  return $self->_GetMainFld($FLD_AUTHOR);
 }   # }}}
 
-# returns hash with format data (i.e. with FLD_FORMAT, $FLD_INDEX, $FLD_FILES keys) 
-# for $format_name 
-sub format($$) { # {{{
+# returns hash with format data (i.e. with FLD_FORMAT, $FLD_INDEX, $FLD_FILES keys)
+# for $format_name
+sub GetFormat($$) { # {{{
   my $self = shift;
   my $format_name = shift;
-  return undef unless $self->_has_control_files();
-  $self->_check_merged();
+  return undef unless $self->_HasControlFiles();
+  $self->_CheckMerged();
   return $self->{'FORMAT_LIST'}->{$format_name};
 } # }}}
 
-# returns status data for $key 
-sub get_status() { # {{{
+# returns status data for $key
+sub GetStatus() { # {{{
   my $self = shift;
   my $key  = shift;
   return $self->{'STATUS_DICT'}->{$key};
 }   # }}}
 
-sub set_status($%) { # {{{
+sub SetStatus($%) { # {{{
   my $self      = shift;
   my %status    = @_;
 
@@ -139,122 +139,17 @@ sub set_status($%) { # {{{
                    or (defined $value and $value ne $oldvalue) );
   }
 
-  $changed ? $self->_write_status_file()
+  $changed ? $self->_WriteStatusDB()
            : Debug("Status of `" . join ("', `", keys %status) . "' in " .
-                    $self->document_id() . " not changed");
+                    $self->GetDocumentID() . " not changed");
 }   # }}}
 
-
-sub _has_control_files() { # {{{
-  my $self = shift;
-  return $self->{'CONTROL_FILES'}
-} # }}}
-
-# reads our status file and sets $self->{'STATUS_DICT'} and sets keys of
-# $self->{'CONTROL_FILES'} 
-sub _read_status_file { # {{{
-  my $self        = shift;
-  my $docid       = $self->document_id();
-  my $status_file = "$DATA_DIR/$docid.status";
-
-  if (-f $status_file) {
-    Debug ("Reading status file `$status_file'");
-    my $status = {};
-    open(S, "<", $status_file)
-      or return Error("Cannot open status file `$status_file' for reading: $!");
-
-    while (<S>) {
-      chomp;
-      next if /^\s*$/o;
-      /^\s*(\S+):\s*"?(.*?)"?\s*$/o
-        or return Warn("Syntax error in status file `$status_file': $_");
-      $$status{$1} = $2;
-    }
-    close(S)
-      or croak "Cannot close status file `$status_file': $!";
-
-    %{$self->{'CONTROL_FILES'}} = map {
-                                    s/^"//;
-                                    s/"$//;
-                                    Debug("Existing control file in status: $_");
-                                    (-f $_) ? ($_ => undef): Warn("Registered control file `$_' no longer exists")
-                                   } split(/\s*,\s*/, $status->{'Control-Files'})
-                                      if $status->{'Control-Files'};
-
-    delete $$status{'Control-Files'};
-    $self->{'STATUS_DICT'} = $status;
-  }
-  $self->{'INVALID'} = 0;
-
-} # }}}
-
-# writes our status file
-sub _write_status_file { # {{{
-  my $self  = shift;
-  my $docid = $self->document_id();
-
-  my $status_file     = "$DATA_DIR/$docid.status";
-  my $tmp_status_file = "$status_file.tmp";
-  Debug ("Writing status information into `$status_file'");
-
-  open(S, ">", $tmp_status_file)
-    or croak "Cannot open status file `$tmp_status_file' for writing: $!";
-
-  my $control_files = '"' . join('", "', sort keys %{$self->{'CONTROL_FILES'}}) . '"';
-  print S "Control-Files: $control_files\n" unless $control_files eq '""';
-
-  my $status = $self->{'STATUS_DICT'};
-  for my $k (sort keys   %$status) {
-    print S "$k: \"$$status{$k}\"\n";
-  }
-  close(S) or croak "Cannot close status file `$tmp_status_file': $!";
-
-  IgnoreSignals();
-  # remove file if it's empty
-  if (-z $tmp_status_file) {
-    unlink $tmp_status_file;
-    unlink $status_file;
-    Debug ("Removing status file `$status_file'");
-  } else {
-    rename $tmp_status_file, $status_file
-      or croak "Can't rename `$tmp_status_file' to `$status_file': $!";
-  }
-  RestoreSignals();
-
-} # }}}
-
-
-# if called without any argument, returns array of control files' names 
-# if called with an argument returns string containing names of the control files
-#  joined with value of the argument
-sub _GetControlFileNames($;$) { # {{{
-  my $self      = shift;
-  my $join_str  = shift;
-
-  my @cfnames = sort keys %{$self->{'CONTROL_FILES'}};
-
-  return @cfnames unless ($join_str);
-  return join($join_str, @cfnames);
-} # }}}
-
-
-# reads and parses all control files mentioned in $self->{'CONTROL_FILES'}
-sub _read_control_files($) { # {{{
-  my $self = shift;
-
-  foreach my $cfname ($self->_GetControlFileNames()) {
-    $self->{'CONTROL_FILES'}->{$cfname} = Debian::DocBase::DocBaseFile->new($cfname, PARSE_FULL)
-      unless $self->{'CONTROL_FILES'}->{$cfname};
-  }
-} # }}}
-
 # displays informations about the document (called by `install-docs -s')
 sub DisplayStatusInformation($) { # {{{
   my $self            = shift;
-  my $docid           = $self->document_id();
+  my $docid           = $self->GetDocumentID();
   my $status_file     = "$DATA_DIR/$docid.status";
   my $var_ctrl_file   = "$VAR_CTRL_DIR/$docid";
-  return unless -f $status_file;
 
   if (-f $var_ctrl_file) {
     if (open(F, '<', $var_ctrl_file)) {
@@ -270,48 +165,49 @@ sub DisplayStatusInformation($) { # {{{
     }
   }
 
-  if (-f $status_file) {
-    if (open(F, '<', $status_file)) {
-      print "\n---status-information---\n";
-      while (<F>) {
-        print $_;
-      }
-      close(F);
-    } else {
-      Warn("Cannot open `$status_file': $!");
-    }
+  print "\n---status-information---\n";
+  foreach my $cf (sort keys %{$self->{'CONTROL_FILES'}} ) {
+    print "Control-File: $cf (changed: ". localtime ($self->{'CONTROL_FILES'}->{$cf}->GetLastChangeTime()) . ")\n";
+  }
+
+  foreach my $key (sort keys %{$self->{'STATUS_DICT'}} ) {
+    print "$key: $self->{'STATUS_DICT'}->{$key}\n";
   }
 } # }}}
 
 sub Register($$) { # {{{
   my $self          = shift;
   my $db_file       = shift;
-  my $db_filename   = $db_file->source_file_name();
+  my $db_filename   = $db_file->GetSourceFileName();
 
   Debug("Registering `$db_filename'");
 
-  if ($db_file->document_id() ne $self->document_id()) {
+  if ($db_file->GetDocumentID() ne $self->GetDocumentID()) {
     delete $self->{'CONTROL_FILES'}->{$db_filename};
+    $db_file->OnRegistered(0);
     return Error("Document id in `$db_filename' does not match our document id (" .
-                  $db_file->document_id() . ' != ' . $self->document_id() . ")");
+                  $db_file->GetDocumentID() . ' != ' . $self->GetDocumentID() . ")");
   }
 
-  if ($db_file->invalid()) {
+  if ($db_file->Invalid()) {
     delete $self->{'CONTROL_FILES'}->{$db_filename};
-    return Warn($db_file->source_file_name() . " contains errors, not registering");
+    $db_file->OnRegistered(0);
+    return Warn($db_file->GetSourceFileName() . " contains errors, not registering");
   }
 
+  $db_file->OnRegistered(1);
   $self->{'CONTROL_FILES'}->{$db_filename} = $db_file;
 } # }}}
 
 sub Unregister($$) { # {{{
   my $self          = shift;
   my $db_file       = shift;
-  my $db_filename   = $db_file->source_file_name();
+  my $db_filename   = $db_file->GetSourceFileName();
 
-  Warn("File `" . $db_filename . "' is not registered, cannot remove")
+  return Warn("File `" . $db_filename . "' is not registered, cannot remove")
     unless exists ($self->{'CONTROL_FILES'}->{$db_filename});
 
+  $self->{'CONTROL_FILES'}->{$db_filename}->OnUnregistered();
   delete $self->{'CONTROL_FILES'}->{$db_filename};
 
 } # }}}
@@ -319,7 +215,12 @@ sub Unregister($$) { # {{{
 sub UnregisterAll($) { # {{{
   my $self          = shift;
 
-  Debug('Unregistering all control files from document `' . $self->document_id() . "'");
+  Debug('Unregistering all control files from document `' . $self->GetDocumentID() . "'");
+
+  foreach my $doc ( values %{$self->{'CONTROL_FILES'}} ) {
+    $doc->OnUnregistered();
+  }
+
 
   $self->{'CONTROL_FILES'} = {};
 } # }}}
@@ -327,14 +228,14 @@ sub UnregisterAll($) { # {{{
 # generate and write new merged control file into /var/lib/doc-base/documents
 sub WriteNewCtrlFile() { # {{{
   my $self     = shift;
-  my $docid    = $self->document_id();
+  my $docid    = $self->GetDocumentID();
   my $tmpfile  = $VAR_CTRL_DIR . "/." . $docid . ".tmp";
   my $file     = $VAR_CTRL_DIR . "/" . $docid;
   my $fld      = undef;
 
-  $self->_check_merged();
+  $self->_CheckMerged();
 
-  if ($self->invalid() || !$self->_has_control_files()) {
+  if ($self->Invalid() || !$self->_HasControlFiles()) {
     if (-e $file)  {
       Debug("Removing control file $file");
       unlink $file or carp "Can't remove $file: $!";
@@ -351,8 +252,6 @@ sub WriteNewCtrlFile() { # {{{
       if $self->{'MAIN_DATA'}->{$fld};
   }
 
-  print F "Control-Files: " . $self->_GetControlFileNames(' ') . "\n" if $self->_has_control_files();
-
   foreach my $format (sort keys %{$self->{'FORMAT_LIST'}}) {
     print F "\n";
     foreach $fld (GetFldKeys($FLDTYPE_FORMAT)) {
@@ -366,8 +265,159 @@ sub WriteNewCtrlFile() { # {{{
   rename $tmpfile, $file or carp "Can't rename $tmpfile to $file: $!";
 } # }}}
 
+# merge contents of all available control files for the document
+#  into $self->{'MAIN_DATA'} and $self->{'FORMAT_LIST'}
+# Fields 'Document' and 'Section' must have the same value in all control files.
+# Value of fields 'Author', 'Abstract', 'Title' is taken from the first control file
+#  in which the value is not empty.
+# Format sections are joined. It's an error if the same format is defined in more
+#  than one control file.
+sub MergeCtrlFiles($) { # {{{
+  my $self    = shift;
+  my $doc_id  = $self->GetDocumentID();
 
-sub _MangleSection($) {
+  $self->_ParseControlFiles();
+
+  $self->{'INVALID'}           = 1;
+  $self->{'MERGED_CTRL_FILES'} = 1;
+  $self->{'MAIN_DATA'}         = {};
+  $self->{'FORMAT_LIST'}       = {};
+
+  foreach my $db_file_name ($self->_GetControlFileNames()) {
+    my $doc_data  = $self->{'CONTROL_FILES'}->{$db_file_name};
+    my $doc_fname = $doc_data->GetSourceFileName();
+
+    if ($doc_data->GetDocumentID() ne $doc_id) {
+      Warn("Document id in `" . $doc_fname ."' does not match our document id (" .
+                  $doc_data->GetDocumentID()  . ' != ' . $self->GetDocumentID() . ")");
+      $self->Unregister($doc_data);
+      next;
+    }
+
+    # merge main sections' fields
+    foreach my $fld (GetFldKeys($FLDTYPE_MAIN)) {
+      my $old_val = $self->{'MAIN_DATA'}->{$fld};
+      my $new_val = $doc_data->GetFldValue($fld);
+      if ($new_val) {
+        $new_val = $self->_MangleSection($new_val) if $fld eq $FLD_SECTION;
+
+        if ($old_val and $old_val ne $new_val and
+            ($fld eq $FLD_DOCUMENT or $fld eq $FLD_SECTION)) {
+            return Error("Error while merging $doc_id with $doc_fname: inconsistent values of $fld");
+        }
+        $self->{'MAIN_DATA'}->{$fld} = $new_val unless $old_val;
+      }
+    }
+
+    # merge formats
+    foreach my $format ($doc_data->GetFormatNames()) {
+      return Error("Error while merging $doc_id with $doc_fname: format $format already defined") if $self->{'FORMAT_LIST'}->{$format};
+      $self->{'FORMAT_LIST'}->{$format} = $doc_data->GetFormat($format);
+    }
+  }
+  return unless  %{$self->{'FORMAT_LIST'}};
+  $self->{'INVALID'}           = 0;
+} # }}}
+
+
+# Save status changes, calls _WriteStatusDB()
+sub SaveStatusChanges($) { # {{{
+  my $self = shift;
+
+  $self->_WriteStatusDB();
+} # }}}
+
+#################################################
+###            PRIVATE FUNCTIONS              ###
+#################################################
+
+# dies with Internal error if document hasn't been merged yet
+sub _CheckMerged($) { # {{{
+  my $self = shift;
+
+  croak "Internal error: Document " . $self->GetDocumentID(). " not yet merged"
+    unless $self->{'MERGED_CTRL_FILES'};
+} # }}}
+
+# returns $fld from $self->{'MAIN_DATA'}
+sub _GetMainFld($$) { # {{{
+  my $self = shift;
+  my $fld  = shift;
+
+  $self->_CheckMerged();
+
+  return "" if $self->Invalid();
+
+  return "" unless $self->{'MAIN_DATA'}->{$fld};
+
+  return $self->{'MAIN_DATA'}->{$fld};
+} # }}}
+
+sub _HasControlFiles() { # {{{
+  my $self = shift;
+  return $self->{'CONTROL_FILES'}
+} # }}}
+
+# reads our status file and sets $self->{'STATUS_DICT'} and sets keys of
+# $self->{'CONTROL_FILES'}
+sub _ReadStatusDB { # {{{
+  my $self        = shift;
+  my $docid       = $self->GetDocumentID();
+  my $data        = Debian::DocBase::DB::GetStatusDB()->GetData($docid);
+
+  if ($data) {
+    my %cf = map { $_ => Debian::DocBase::DocBaseFile->new($_) } keys %{$data->{'CF'}};
+    $self->{'CONTROL_FILES'}  = \%cf;
+    $self->{'STATUS_DICT'}    = $data->{'SD'};
+  } else {
+    $self->{'CONTROL_FILES'} = {};
+    $self->{'STATUS_DICT'}   = {};
+  };
+  $self->{'INVALID'} = 0;
+
+} # }}}
+
+# writes our status file
+sub _WriteStatusDB { # {{{
+  my $self  = shift;
+  my $docid = $self->GetDocumentID();
+
+  if (%{$self->{'CONTROL_FILES'}} or %{$self->{'STATUS_DICT'}}) {
+    my %cf = map { $_ => undef }  keys %{$self->{'CONTROL_FILES'}};
+
+    my $data = { 'CF' => \%cf,
+                 'SD' => $self->{'STATUS_DICT'}
+               };
+   Debian::DocBase::DB::GetStatusDB()->PutData($docid, $data);
+  } else {
+   Debian::DocBase::DB::GetStatusDB()->RemoveData($docid);
+  }
+} # }}}
+
+# if called without any argument, returns array of control files' names
+# if called with an argument returns string containing names of the control files
+#  joined with value of the argument
+sub _GetControlFileNames($;$) { # {{{
+  my $self      = shift;
+  my $join_str  = shift;
+
+  my @cfnames = sort keys %{$self->{'CONTROL_FILES'}};
+
+  return @cfnames unless ($join_str);
+  return join($join_str, @cfnames);
+} # }}}
+
+# reads and parses all control files mentioned in $self->{'CONTROL_FILES'}
+sub _ParseControlFiles($) { # {{{
+  my $self = shift;
+
+  foreach my $cfname ($self->_GetControlFileNames()) {
+    croak "Internal error: $cfname not created\n" unless $self->{'CONTROL_FILES'}->{$cfname};
+    $self->{'CONTROL_FILES'}->{$cfname}->Parse();
+  }
+} # }}}
+
+sub _MangleSection($) { # {{{
   my $self      = shift;
   my $section   = shift;
 
@@ -380,78 +430,17 @@ sub _MangleSection($) {
 
   my @sect_comps = split (/\/+/, $section);
   my $result     = "";
-  my $from_map   = 0;
 
   while ($#sect_comps > -1) {
     my $tmp   =  shift(@sect_comps);
     $result   =  ($result) ? $result . "/" .  $tmp : $tmp;
 
-    $tmp      = $section_map{lc $result};
-    $result   = $tmp if $tmp;
+    $tmp      = lc $result;
+    $result   = $section_map{$tmp} if exists $section_map{$tmp};
   }
 
   return $result if $result;
   return "Unknown";
-}
-
-# merge contents of all available control files for the document
-#  into $self->{'MAIN_DATA'} and $self->{'FORMAT_LIST'}
-# Fields 'Document' and 'Section' must have the same value in all control files.
-# Value of fields 'Author', 'Abstract', 'Title' is taken from the first control file 
-#  in which the value is not empty.
-# Format sections are joined. It's an error if the same format is defined in more
-#  than one control file.
-sub MergeCtrlFiles($) { # {{{
-  my $self    = shift;
-
-  $self->_read_control_files();
-
-  $self->{'INVALID'}           = 1;
-  $self->{'MERGED_CTRL_FILES'} = 1;
-  $self->{'MAIN_DATA'}         = {};
-  $self->{'FORMAT_LIST'}       = {};
-
-  foreach my $db_file_name ($self->_GetControlFileNames()) {
-    my $doc_data = $self->{'CONTROL_FILES'}->{$db_file_name};
-
-    if ($doc_data->document_id() ne $self->document_id()) {
-      Warn("Document id in `" . $doc_data->source_file_name() ."' does not match our document id (" .
-                  $doc_data->document_id() . ' != ' . $self->document_id() . ")");
-      $self->Unregister($doc_data);
-      next;
-    }
-
-    # merge main sections' fields
-    foreach my $fld (GetFldKeys($FLDTYPE_MAIN)) {
-      my $old_val = $self->{'MAIN_DATA'}->{$fld};
-      my $new_val = $doc_data->GetFldValue($fld);
-      if ($new_val) {
-        $new_val = $self->_MangleSection($new_val) if $fld eq $FLD_SECTION;
-        
-        if ($old_val and $old_val ne $new_val and
-            ($fld eq $FLD_DOCUMENT or $fld eq $FLD_SECTION)) {
-            return Error("Error while merging: inconsistent values of $fld");
-          }
-        $self->{'MAIN_DATA'}->{$fld} = $new_val unless $old_val;
-      }
-    }
-
-    # merge formats
-    foreach my $format ($doc_data->GetFormatNames()) {
-      return Error("format $format already defined") if $self->{'FORMAT_LIST'}->{$format};
-      $self->{'FORMAT_LIST'}->{$format} = $doc_data->format($format);
-    }
-  }
-  return unless  %{$self->{'FORMAT_LIST'}};
-  $self->{'INVALID'}           = 0;
-} # }}}
-
-
-# Save status changes, calls _write_status_file()
-sub SaveStatusChanges($) { # {{{
-  my $self = shift;
-
-  $self->_write_status_file();
 } # }}}
 
 1;
